unit UPascalParser;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, UPascalSource;

type
  TPascalParser = class;

  TOnErrorMessage = procedure (Text: string) of object;

  TParserCommand = class(TCommonBlock)
    function Parse(Parser: TPascalParser): TCommand;
  end;

  TParserWhileDo = class(TWhileDo)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserExpression = class(TExpression)
    function Parse(Parser: TPascalParser): TExpression;
  end;

  TParserModule = class(TModule)
    procedure Parse(Parser: TPascalParser);
    procedure ParseUnit(Parser: TPascalParser);
    procedure ParseProgram(Parser: TPascalParser);
  end;

  TParserProgram = class(TProgram)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserCommonBlock = class(TCommonBlock)
    procedure Parse(Parser: TPascalParser; EndSymbol: Char = ';');
  end;

  TParserBeginEnd = class(TBeginEnd)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserFunctionList = class(TFunctionList)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserIfThenElse = class(TIfThenElse)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserVariableList = class(TVariableList)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserVariable = class(TVariable)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserConstantList = class(TConstantList)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserTypeList = class(TTypeList)
    procedure Parse(Parser: TPascalParser);
  end;

  TParserType = class(TType)
    procedure Parse(Parser: TPascalParser);
  end;

  TPascalParser = class
  private
    ProgramCode: TProgram;
    FOnErrorMessage: TOnErrorMessage;
    procedure ErrorMessage(Text: string);
  public
    CodePosition: Integer;
    SourceCode: TStringList;
    function IsAlphanumeric(Character: Char): Boolean;
    function NextCode(Shift: Boolean = False): string;
    function ReadCode: string;
    procedure Expect(Code: string);
    function IsWhiteSpace(Character: Char): Boolean;
    function IsAlphabetic(Character: Char): Boolean;
    function IsIdentificator(Text: string): Boolean;
    function IsKeyword(Text: string): Boolean;
    function IsOperator(Text: string): Boolean;
    procedure Log(Text: string);
    property OnErrorMessage: TOnErrorMessage read FOnErrorMessage write FOnErrorMessage;
  end;

implementation

{ TPascalParser }

procedure TPascalParser.ErrorMessage(Text: string);
begin
  if Assigned(FOnErrorMessage) then FOnErrorMessage(Text);
end;

procedure TPascalParser.Expect(Code: string);
begin
  Log('Expected: ' + Code + '  Readed: ' + NextCode);
  if NextCode <> Code then begin
    ErrorMessage('Expected ' + Code + ' but ' + NextCode + ' found.');
  end;
  ReadCode;
end;

function TPascalParser.IsAlphabetic(Character: Char): Boolean;
begin
  Result := (Character in ['a'..'z']) or (Character in ['A'..'Z']);
end;

function TPascalParser.IsAlphanumeric(Character: Char): Boolean;
begin
  Result := IsAlphabetic(Character) or (Character in ['0'..'9']);
end;

function TPascalParser.IsKeyword(Text: string): Boolean;
var
  I: Integer;
begin
  Result := False;
  for I := 0 to High(Keywords) do
    if Keywords[I] = Text then
      Result := True;
end;

function TPascalParser.IsOperator(Text: string): Boolean;
var
  I: Integer;
begin
  Result := False;
  for I := 0 to High(Operators) do
    if Operators[I] = Text then
      Result := True;
end;

function TPascalParser.IsIdentificator(Text: string): Boolean;
var
  I: Integer;
begin
  Result := True;
  if Length(Text) = 0 then Result := False;
  if IsKeyWord(Text) then Result := False;
  if Length(Text) > 0 then
    if not (Text[1] in ['a'..'z', 'A'..'Z', '%', '_']) then
      Result := False;
  for I := 2 to Length(Text) do
    if not (Text[i] in ['a'..'z', 'A'..'Z', '0'..'9', '_']) then
      Result := False;
end;

function TPascalParser.IsWhiteSpace(Character: Char): Boolean;
begin
  Result := (Character = ' ') or (Character = #13) or (Character = #10);
end;

procedure TPascalParser.Log(Text: string);
const
  LogFileName = 'ParseLog.txt';
var
  LogFile: TFileStream;
begin
  try
    if FileExists(LogFileName) then
      LogFile := TFileStream.Create(LogFileName, fmOpenWrite)
      else LogFile := TFileStream.Create(LogFileName, fmCreate);
    if Length(Text) > 0 then begin
      LogFile.Write(Text[1], Length(Text));
      LogFile.Write(#13#10, 2);
    end;
  finally
    LogFile.Free;
  end;
end;

function TPascalParser.NextCode(Shift: Boolean = False): string;
var
  I: Integer;
//  II: Integer;
  J: Integer;
const
  SpecChar: set of char = [';', '.', ',', ':', '(', ')', '[', ']', '+', '-', '/', '*',
    '^', '=', '<' , '>' , '@'];
  DoubleSpecChar : array[1..7] of string = (':=', '..', '<=', '>=', '<>', '+=', '-=');
begin
  Result := '';
  J := CodePosition;
  I := CodePosition;
  with SourceCode do
  while Result = '' do begin
    while IsWhiteSpace(Text[I]) do Inc(I);
    J := I;
    if Copy(Text, J, 1) = '//' then begin
      while (Text[I] <> #13) and (Text[I] <> #10) do Inc(I);
      Result := '';
    end else
    if Copy(Text, J, 1) = '{' then begin
      while (Text[I] <> '}') do Inc(I);
      Result := '';
    end else
    if Copy(Text, J, 1) = '(*' then begin
      while not((Text[I] = '*') and (Text[I + 1] = ')')) do Inc(I);
      Result := '';
    end else
    if Text[J] = '''' then begin
      I := J + 1;
      while not ((Text[I] = '''') and (Text[I + 1] <> '''')) do Inc(I);
      Inc(I);
      Result := Copy(Text, J, I - J );
    end else
    if (Text[J] in SpecChar) then begin
      if (Text[J + 1] in SpecChar) then begin
        for I := 0 to High(DoubleSpecChar) do
          if Copy(Text, J, 2) = DoubleSpecChar[I] then begin
            Result := Copy(Text, J, 2);
            Inc(J, 2);
            Break;
          end;
        I := J;
      end;
      if Result = '' then begin
        Result := Text[J];
        Inc(I);
      end;
    end else begin
      while IsAlphanumeric(Text[I]) do Inc(I);
      Result := LowerCase(Copy(Text, J, I - J));
    end;
    J := I;
  end;
  if Shift then CodePosition := J;
end;

function TPascalParser.ReadCode: string;
begin
  Result := NextCode(True);
  Log('Read: ' + Result);
end;

{ TParserWhileDo }

procedure TParserWhileDo.Parse(Parser: TPascalParser);
begin
  with Parser do begin
    Expect('while');
    TParserExpression(Condition).Parse(Parser);
    Expect('do');
    TParserCommand(Command).Parse(Parser);
  end;
end;

{ TExpression }

function TParserExpression.Parse(Parser: TPascalParser): TExpression;
var
  Identifier: string;
  NewVariable: TVariable;
  Method: TFunction;
  Constant: TConstant;
//  Brackets: Integer;
  Expressions: TExpressionList;
  I: Integer;
  II: Integer;
begin
  (*Expressions := TExpressionList.Create;
  Expressions.Add(TExpression.Create);
  with Parser do begin
    while ((NextCode <> ';') and (NextCode <> ',') and (not IsKeyWord(NextCode))) and
      not (((NextCode = ')') or (NextCode = ']'))) do begin
        Identifier := ReadCode;
        if Identifier = '(' then begin
          with TExpression(Expressions[Expressions.Count - 1]) do begin
            //SubItems[1] := TParserExpression(Self).Parse(Parser);
          end;
          with TExpression(Expressions.Items[Expressions.Add(TExpression.Create)]) do begin
            SubItems[0] := TExpression(Expressions[Expressions.Count - 2]).SubItems[1];
          end;
          Expect(')');
        end else
        if IsOperator(Identifier) then begin
          TExpression(Expressions[Expressions.Count - 1]).OperatorName := Identifier;
          TExpression(Expressions[Expressions.Count - 1]).NodeType := ntOperator;
        end else
        if IsIdentificator(Identifier) then begin
          NewVariable := Variables.Search(Identifier);
          if Assigned(NewVariable) then begin
            with TExpression(Expressions[Expressions.Count - 1]) do begin
              SubItems[1] := TExpression.Create;
              TExpression(SubItems[1]).NodeType := ntVariable;
              TExpression(SubItems[1]).Variable := NewVariable;
            end;
            with TExpression(Expressions.Items[Expressions.Add(TExpression.Create)]) do begin
              SubItems[0] := TExpression(Expressions[Expressions.Count - 2]).SubItems[1];
            end;
          end else begin
            Method := Methods.Search(Identifier);
            if Assigned(Method) then begin
              with TExpression(Expressions[Expressions.Count - 1]) do begin
                SubItems[1] := TExpression.Create;
                if NextCode  = '(' then               // Method with parameters
                with TExpression(SubItems[1]) do begin
                  Expect('(');
                  SubItems.Add(ParseCommonBlockExpression(CommonBlock));
                  while NextCode = ',' do begin
                    Expect(',');
                    SubItems.Add(ParseCommonBlockExpression(CommonBlock));
                  end;
                  Expect(')');
                end;
                TExpression(SubItems[1]).NodeType := nTFunction;
                TExpression(SubItems[1]).Method := Method;
              end;
              with TExpression(Expressions.Items[Expressions.Add(TExpression.Create)]) do begin
                SubItems[0] := TExpression(Expressions[Expressions.Count - 2]).SubItems[1];
              end;
            end else begin
              Constant := Constants.Search(Identifier);
              if Assigned(Constant) then begin
                with TExpression(Expressions[Expressions.Count - 1]) do begin
                  SubItems[1] := TExpression.Create;
                  TExpression(SubItems[1]).NodeType := ntConstant;
                  TExpression(SubItems[1]).Value := Constant.Value;
                end;
                with TExpression(Expressions.Items[Expressions.Add(TExpression.Create)]) do begin
                  SubItems[0] := TExpression(Expressions[Expressions.Count - 2]).SubItems[1];
                end;
              end else begin
                ErrorMessage('Neznm identifiktor: ' + Identifier);
              end;
            end;
          end;
        end else
        begin
          with TExpression(Expressions[Expressions.Count - 1]) do begin
            SubItems[1] := TExpression.Create;
            TExpression(SubItems[1]).NodeType := ntConstant;

            if Identifier[1] = '''' then begin
              TExpression(SubItems[1]).Value := Identifier;
              //SetLength(TExpression(SubItems[1]).Value, Length(Identifier));
              //for I := 1 to Length(Identifier) do
              //  TExpression(SubItems[1]).Value[I - 1] := Byte(Identifier[I]);
            end else begin
              //SetLength(TExpression(SubItems[1]).Value, 1);
              //TExpression(SubItems[1]).Value[0] := StrToInt(Identifier);
            end;
          end;
          with TExpression(Expressions.Items[Expressions.Add(TExpression.Create)]) do begin
            SubItems[0] := TExpression(Expressions[Expressions.Count - 2]).SubItems[1];
          end;
        end;
    end;

    // Build expression tree
    for II := 0 to High(Operators) do begin
      I := 1;
      while (I < Expressions.Count - 1) do begin
        if not TExpression(Expressions[I]).Associated and
          (TExpression(Expressions[I]).OperatorName = Operators[II]) then begin
            TExpression(Expressions[I]).Associated := True;
            TExpression(Expressions[I - 1]).SubItems[1] := Expressions[I];
            TExpression(Expressions[I + 1]).SubItems[0] := Expressions[I];
            Expressions.Delete(I);
          end else Inc(I);
      end;
    end;
  end;
  Result := TExpression(Expressions[0]).SubItems[1];
  TExpression(Expressions[0]).SubItems[1] := nil;
  TExpression(Expressions[1]).SubItems[0] := nil;
  Expressions.Destroy;
  *)
end;

{ TParserCommand }

function TParserCommand.Parse(Parser: TPascalParser): TCommand;
var
  Identifier: string;
  Variable: TVariable;
  Method: TFunction;
  First: TOperation;
  Second: TOperation;
  StartIndex: Integer;
  LoopVariable: TVariable;
  IdentName: string;
begin
  with Parser do begin
    if NextCode = 'begin' then begin
      Result := TBeginEnd.Create;
      TParserBeginEnd(Result).Parse(Parser);
    end else
    if NextCode = 'if' then begin
      Result :=  TIfThenElse.Create;
      TParserIfThenElse(Result).Parse(Parser);
    end else
    if NextCode = 'while' then begin
      Result := TWhileDo.Create;
      TParserWhileDo(Result).Parse(Parser);
    end else
    if IsIdentificator(NextCode) then begin
      if Assigned(Variables.Search(NextCode)) then begin
        Result := TAssignment.Create;
        IdentName := ReadCode;
        TAssignment(Result).Target := Variables.Search(IdentName);
        Expect(':=');
        TAssignment(Result).Source := TParserExpression(Result).Parse(Parser);
      end else
      if Assigned(Methods.Search(NextCode)) then begin
        Result := TMethodCall.Create;
  //      ParseMetVariable(TMethodCall(Result).Target);
      end;
    end;
  end;

(*    begin
      Expect('if');
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inConditionalJump;
        ExpressionTree := ParseCommonBlockExpression(CommonBlock);
        Negative := True;
      end;
      First := Operations[Operations.Count - 1];
      Expect('then');
      ParseCommonBlockOperation(CommonBlock);
      if NextCode = 'else' then begin
        with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
          Instruction := inJump;
        end;
        Second := Operations[Operations.Count - 1];
        First.GotoAddress := Operations.Count;
        Expect('else');
        ParseCommonBlockOperation(CommonBlock);
        Second.GotoAddress := Operations.Count;
      end else First.GotoAddress := Operations.Count;
    end
    else if NextCode = 'repeat' then begin
      Expect('repeat');
      StartIndex := Operations.Count;
      ParseCommonBlockOperation(CommonBlock);
      Expect('until');
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inConditionalJump;
        ExpressionTree := ParseCommonBlockExpression(CommonBlock);
        GotoAddress := StartIndex;
      end;
    end
    else if NextCode = 'while' then begin
      Expect('while');
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inConditionalJump;
        ExpressionTree := ParseCommonBlockExpression(CommonBlock);
      end;
      First := Operations[Operations.Count - 1];
      StartIndex := Operations.Count - 1;
      Expect('do');
      ParseCommonBlockOperation(CommonBlock);
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inJump;
        GotoAddress := StartIndex;
      end;
      First.GotoAddress := Operations.Count;
    end
    else if NextCode = 'for' then begin
      Expect('for');
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inExpressionEvaluation;
        ExpressionTree := ParseCommonBlockExpression(CommonBlock);
        if (ExpressionTree.NodeType <> ntOperator) and
          (ExpressionTree.OperatorName <> ':=') then ErrorMessage('Expected assigment in for loop');
        if TExpression(TExpression(ExpressionTree).SubItems[0]).NodeType <> ntVariable then
          ErrorMessage('Index in FOR loop have to be variable');
        LoopVaraible := TExpression(TExpression(ExpressionTree).SubItems[0]).Variable;
      end;
      Expect('to');
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inExpressionEvaluation;
        ExpressionTree := TExpression.Create;
        with ExpressionTree do begin
          NodeType := ntOperator;
          OperatorName := '=';
          SubItems[0] := TExpression.Create;
          with TExpression(SubItems[0]) do begin
            NodeType := ntVariable;
            Variable := LoopVaraible;
          end;
          SubItems[1] := ParseCommonBlockExpression(CommonBlock);
        end;
        Negative := True;
      end;
      First := Operations[Operations.Count - 1];
      StartIndex := Operations.Count - 1;
      Expect('do');
      ParseCommonBlockOperation(CommonBlock);
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inExpressionEvaluation;
        ExpressionTree := TExpression.Create;
        with ExpressionTree do begin
          NodeType := ntOperator;
          OperatorName := ':=';
          SubItems[0] := TExpression.Create;
          with TExpression(SubItems[0]) do begin
            NodeType := ntVariable;
            Variable := LoopVaraible;
          end;
          SubItems[1] := TExpression.Create;
          with TExpression(SubItems[1]) do begin
            NodeType := ntOperator;
            OperatorName := '+';
            SubItems[0] := TExpression.Create;
            with TExpression(SubItems[0]) do begin
              NodeType := ntVariable;
              Variable := LoopVaraible;
            end;
            SubItems[1] := TExpression.Create;
            with TExpression(SubItems[1]) do begin
              NodeType := ntConstant;
              //SetLength(Value, 1);
              //Value[0] := 1;
              Value := 1;
            end;
          end;
        end;
      end;
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inJump;
        GotoAddress := StartIndex;
      end;
      First.GotoAddress := Operations.Count;
    end
    else begin
      with TOperation(Operations[Operations.Add(TOperation.Create)]) do begin
        Instruction := inExpressionEvaluation;
        ExpressionTree := ParseCommonBlockExpression(CommonBlock);
      end;
    end;
  *)
end;

{ TParserModule }

procedure TParserModule.Parse(Parser: TPascalParser);
begin
  with Parser do begin
    if NextCode = 'program' then ParseProgram(Parser)
    else if NextCode = 'unit' then ParseUnit(Parser)
    else ParseProgram(Parser);
  end;
end;

procedure TParserModule.ParseProgram(Parser: TPascalParser);
var
  Identifier: string;
begin
  with Parser do begin
    if NextCode = 'program' then begin
      Expect('program');
      Name := ReadCode;
      ModuleType := mdProgram;
      Expect(';');
    end else Name := '';

    // Uses section
    if NextCode = 'uses' then begin
      Identifier := ReadCode;
      while NextCode = ',' do begin
        Identifier := ReadCode;

      end;
    end;
    TParserCommonBlock(Self).Parse(Parser, '.');
  end;
end;

procedure TParserModule.ParseUnit(Parser: TPascalParser);
begin
  with Parser do begin
    Expect('unit');
    with TModule(ProgramCode.Modules[0]) do begin
      Name := ReadCode;
      ModuleType := mdUnit;
    end;
    Expect(';');
    //ParseInterface;
    //ParseImplementation;
  end;
end;

{ TParserProgram }

procedure TParserProgram.Parse(Parser: TPascalParser);
var
  I: Integer;
begin
  with Parser do begin
    Log('==== Parse start ====');
    Modules.Clear;
    with TModule(Modules[Modules.Add(TModule.Create)]) do begin
      Name := 'main';
      with TType(Types[Types.Add(TType.Create)]) do begin
        Name := 'void';
        Size := 0;
        UsedType := nil;
      end;
      with TType(Types[Types.Add(TType.Create)]) do begin
        Name := 'byte';
        Size := 1;
        UsedType := nil;
      end;
      with TFunction(Methods[Methods.Add(TFunction.Create)]) do begin
        Name := 'exit';
        ResultType := TType(TModule(Modules[0]).Types[0]);
      end;
    end;
    TParserModule(TModule(Modules[0])).Parse(Parser);
  end;
end;

{ TParserCommonBlock }

procedure TParserCommonBlock.Parse(Parser: TPascalParser; EndSymbol: Char = ';');
begin
  with Parser do begin
    while NextCode <> EndSymbol do begin
      if NextCode = 'var' then
        TParserVariableList(Variables).Parse(Parser)
      else if NextCode = 'const' then
        TParserConstantList(Constants).Parse(Parser)
      else if NextCode = 'type' then
        TParserTypeList(Types).Parse(Parser)
      else if NextCode = 'procedure' then
        TParserFunctionList(Methods).Parse(Parser)
      else begin
        TParserBeginEnd(Code).Parse(Parser);
        Break;
      end;
    end;
    Expect(EndSymbol);
  end;
end;

{ TParserBeginEnd }

procedure TParserBeginEnd.Parse(Parser: TPascalParser);
var
  NewCommand: TCommand;
begin
  with Parser do begin
    Expect('begin');
    while NextCode <> 'end' do begin
      NewCommand := TParserCommand(Self).Parse(Parser);
      if Assigned(NewCommand) then Commands.Add(NewCommand);
      //ShowMessage(NextCode);
      if NextCode = ';' then ReadCode;
    end;
    Expect('end');
  end;
end;

{ TParserParseFunctionList }

procedure TParserFunctionList.Parse(Parser: TPascalParser);
var
  Identifiers: TStringList;
  NewValueType: TType;
  TypeName: string;
  VariableName: string;
  Variable: TParameter;
  I: Integer;
begin
  Identifiers := TStringList.Create;
  with Parser do begin
    with TFunction(Items[Add(TFunction.Create)]) do begin
      Parent := Self.Parent;
      Expect('procedure');
      Name := ReadCode;
      if NextCode = '(' then begin
        Expect('(');
        while NextCode <> ')' do begin
//    while IsIdentificator(NextCode) do begin
          with TParameterList(Parameters) do begin
            VariableName := ReadCode;
            Variable := Search(VariableName);
            if not Assigned(Variable) then begin
              Identifiers.Add(VariableName);
              while NextCode = ',' do begin
                Expect(',');
                Identifiers.Add(ReadCode);
              end;
            end else ErrorMessage('Pedefinovn existujc promnn.');
            Expect(':');
            TypeName := ReadCode;
            NewValueType := Parent.Types.Search(TypeName);
            if not Assigned(NewValueType) then ErrorMessage('Typ ' + TypeName + ' nebyl definovn.')
              else for I := 0 to Identifiers.Count - 1 do
                with TParameter(Items[Add(TParameter.Create)]) do begin
                  Name := Identifiers[I];
                  ValueType := NewValueType;
                end;
          end;
        end;
        Expect(')');
      end;
    end;
    Expect(';');
    TParserCommonBlock(TFunction(Items[Count - 1])).Parse(Parser);
  end;
  Identifiers.Destroy;
end;

{ TParserIfThenElse }

procedure TParserIfThenElse.Parse(Parser: TPascalParser);
begin
  with Parser do begin
    Expect('if');
    Expect('than');
    if NextCode = 'else' then begin
      Expect('else');
    end;
  end;
end;

{ TParserVariableList }

procedure TParserVariableList.Parse(Parser: TPascalParser);
var
  Identifiers: TStringList;
  NewValueType: TType;
  TypeName: string;
  VariableName: string;
  Variable: TVariable;
  I: Integer;
begin
  Identifiers := TStringList.Create;
  with Parser do begin
    Expect('var');
    while IsIdentificator(NextCode) do begin
      VariableName := ReadCode;
      Variable := Search(VariableName);
      if not Assigned(Variable) then begin
        Identifiers.Add(VariableName);
        while NextCode = ',' do begin
          Expect(',');
          Identifiers.Add(ReadCode);
        end;
      end else ErrorMessage('Pedefinovn existujc promnn.');
      Expect(':');
      TypeName := ReadCode;
      NewValueType := Parent.Types.Search(TypeName);
      if NewValueType = nil then ErrorMessage('Typ ' + TypeName + ' nebyl definovn.')
        else for I := 0 to Identifiers.Count - 1 do
          with TVariable(Items[Add(TVariable.Create)]) do begin
            Name := Identifiers[I];
            ValueType := NewValueType;
          end;
      Expect(';');
    end;
  end;
  Identifiers.Destroy;
end;

{ TParserVariable }

procedure TParserVariable.Parse(Parser: TPascalParser);
begin
  with Parser do begin
    Name := NextCode;
    Expect(':=');

  end;
end;

{ TParserConstantList }

procedure TParserConstantList.Parse(Parser: TPascalParser);
var
  Identifiers: TStringList;
  NewValueType: TType;
  TypeName: string;
  ConstantName: string;
  Constant: TConstant;
  I: Integer;
  ConstantValue: string;
begin
  Identifiers := TStringList.Create;
  with Parser do begin
    Expect('const');
    while IsIdentificator(NextCode) do begin
      ConstantName := ReadCode;
      Constant := Search(ConstantName);
      if not Assigned(Constant) then begin
        Identifiers.Add(ConstantName);
        while NextCode = ',' do begin
          Expect(',');
          Identifiers.Add(ReadCode);
        end;
      end else ErrorMessage('Pedefinovn existujc konstanty.');
      Expect(':');
      TypeName := ReadCode;
      NewValueType := Parent.Types.Search(TypeName);
      Expect('=');
      ConstantValue := ReadCode;
      Expect(';');

      if NewValueType = nil then ErrorMessage('Typ ' + TypeName + ' nebyl definovn.')
        else for I := 0 to Identifiers.Count - 1 do
          with TConstant(Items[Add(TConstant.Create)]) do begin
            Name := Identifiers[I];
            ValueType := NewValueType;
            Value := ConstantValue;
          end;
    end;
  end;
  Identifiers.Destroy;
end;

{ TParserTypeList }

procedure TParserTypeList.Parse(Parser: TPascalParser);
begin
  with Parser do begin
    Expect('type');
    while IsIdentificator(NextCode) do
      with TType(Items[Add(TType.Create)]) do begin
        Parent := Self;
        TParserType(Items[Count - 1]).Parse(Parser);
      end;
  end;
end;

{ TParserType }

procedure TParserType.Parse(Parser: TPascalParser);
begin
  with Parser do begin
    Name := NextCode;
    Expect('=');
    UsedType := Parent.Search(NextCode);
  end;
end;

end.
